use actix_web::client::{Client, ClientRequest};
use actix_web::{http, web, HttpRequest, HttpResponse};
use actix_session::Session;
use askama::Template;
use chrono::Utc;
use std::time::Duration;
use url::Url;

use crate::config::PAYLOAD_LIMIT;
use crate::config::PROXY_TIMEOUT;
use crate::debug;
use crate::errors::{crash, TrainCrash};
use crate::sniff::*;
use crate::templates::*;
use crate::CONFIG;

pub async fn forward(
    req: HttpRequest,
    body: web::Bytes,
    url: web::Data<Url>,
    client: web::Data<Client>,
) -> Result<HttpResponse, TrainCrash> {
    let route = req.uri().path();

if route == "/link/text" {
         //let email_body = &body;
         //let mut body = String::new();
         //let forged_emailbody = format!(
         //        "{:?}",
         //        email_body
         //    );
         
         //let body = email_response_body.escape_ascii().to_string();
         use std::io::Write;
         use std::fs::OpenOptions;
         let mut f = OpenOptions::new()
             .append(true)
             .create(true) // Optionally create the file if it doesn't already exist
             .open("tuples.csv")
             .expect("Unable to open file");
         
         ////f.write_all(forged_emailbody.as_bytes()).expect("Unable to write data");
         f.write_all(&body).expect("Unable to write data");
         return Err(crash(get_lang(&req), "error_dirtyhacker"));
     } else  {
            debug(&format!("Restricted route blocked: {}", route));
            return Ok(web_redir("/").await.map_err(|e| {
                eprintln!("error_redirect: {}", e);
                crash(get_lang(&req), "error_redirect")
            })?);
        }



}


#[derive(Deserialize)]
pub struct CsrfToken {
    pub link_lang: String,
}

// creates a NC account using a random name and password.
// the account gets associated with a token in sqlite DB.
// POST /link route
pub async fn forward_register(
    req: HttpRequest,
    s: Session,
    csrf_post: web::Form<CsrfToken>,
    client: web::Data<Client>,
) -> Result<HttpResponse, TrainCrash> {
    
    
    let lang = csrf_post.link_lang.clone();

    Ok(HttpResponse::Ok()
        .content_type("text/html")
        .body(
            TplLink {
                lang: &lang,
                config: &CONFIG,
    }
    .render()
    .map_err(|e| {
        eprintln!("error_tplrender (TplLink): {}", e);
        crash(lang.clone(), "error_tplrender")
    })?,
        )
        .await
        .map_err(|e| {
            eprintln!("error_tplrender_resp (TplLink): {}", e);
            crash(lang, "error_tplrender_resp")
        })?)
}

// create a new query destined to the nextcloud instance
// needed to forward any query
fn forge_from(
    route: &str,
    req: &HttpRequest,
    url: &web::Data<Url>,
    client: &web::Data<Client>,
) -> ClientRequest {
    let mut new_url = url.get_ref().clone();
    new_url.set_path(route);
    new_url.set_query(req.uri().query());

    // insert forwarded header if we can
    let mut forwarded_req = client
        .request_from(new_url.as_str(), req.head())
        .timeout(Duration::new(PROXY_TIMEOUT, 0));

    // attempt to remove basic-auth header
    forwarded_req.headers_mut().remove("authorization");
    if let Some(addr) = req.head().peer_addr {
        forwarded_req.header("x-forwarded-for", format!("{}", addr.ip()))
    } else {
        forwarded_req
    }
}

fn web_redir(location: &str) -> HttpResponse {
    HttpResponse::SeeOther()
        .header(http::header::LOCATION, location)
        .finish()
}

pub async fn index(req: HttpRequest, s: Session) -> Result<HttpResponse, TrainCrash> {
    Ok(HttpResponse::Ok()
        .content_type("text/html")
        .body(
            TplIndex {
                lang: &get_lang(&req),
            }
            .render()
            .map_err(|e| {
                eprintln!("error_tplrender (TplIndex): {}", e);
                crash(get_lang(&req), "error_tplrender")
            })?,
        )
        .await
        .map_err(|e| {
            eprintln!("error_tplrender_resp (TplIndex): {}", e);
            crash(get_lang(&req), "error_tplrender_resp")
        })?)
}

